# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from savvihub_client.configuration import Configuration


class ResponseVolumeSourceDataset(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'dataset': 'ResponseDataset',
        'dataset_id': 'int',
        'snapshot': 'ResponseSnapshot',
        'snapshot_id': 'int',
        'sub_path': 'str'
    }

    attribute_map = {
        'dataset': 'dataset',
        'dataset_id': 'dataset_id',
        'snapshot': 'snapshot',
        'snapshot_id': 'snapshot_id',
        'sub_path': 'sub_path'
    }

    def __init__(self, dataset=None, dataset_id=None, snapshot=None, snapshot_id=None, sub_path=None, local_vars_configuration=None):  # noqa: E501
        """ResponseVolumeSourceDataset - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._dataset = None
        self._dataset_id = None
        self._snapshot = None
        self._snapshot_id = None
        self._sub_path = None
        self.discriminator = None

        if dataset is not None:
            self.dataset = dataset
        if dataset_id is not None:
            self.dataset_id = dataset_id
        if snapshot is not None:
            self.snapshot = snapshot
        if snapshot_id is not None:
            self.snapshot_id = snapshot_id
        if sub_path is not None:
            self.sub_path = sub_path

    @property
    def dataset(self):
        """Gets the dataset of this ResponseVolumeSourceDataset.  # noqa: E501


        :return: The dataset of this ResponseVolumeSourceDataset.  # noqa: E501
        :rtype: ResponseDataset
        """
        return self._dataset

    @dataset.setter
    def dataset(self, dataset):
        """Sets the dataset of this ResponseVolumeSourceDataset.


        :param dataset: The dataset of this ResponseVolumeSourceDataset.  # noqa: E501
        :type dataset: ResponseDataset
        """

        self._dataset = dataset

    @property
    def dataset_id(self):
        """Gets the dataset_id of this ResponseVolumeSourceDataset.  # noqa: E501


        :return: The dataset_id of this ResponseVolumeSourceDataset.  # noqa: E501
        :rtype: int
        """
        return self._dataset_id

    @dataset_id.setter
    def dataset_id(self, dataset_id):
        """Sets the dataset_id of this ResponseVolumeSourceDataset.


        :param dataset_id: The dataset_id of this ResponseVolumeSourceDataset.  # noqa: E501
        :type dataset_id: int
        """

        self._dataset_id = dataset_id

    @property
    def snapshot(self):
        """Gets the snapshot of this ResponseVolumeSourceDataset.  # noqa: E501


        :return: The snapshot of this ResponseVolumeSourceDataset.  # noqa: E501
        :rtype: ResponseSnapshot
        """
        return self._snapshot

    @snapshot.setter
    def snapshot(self, snapshot):
        """Sets the snapshot of this ResponseVolumeSourceDataset.


        :param snapshot: The snapshot of this ResponseVolumeSourceDataset.  # noqa: E501
        :type snapshot: ResponseSnapshot
        """

        self._snapshot = snapshot

    @property
    def snapshot_id(self):
        """Gets the snapshot_id of this ResponseVolumeSourceDataset.  # noqa: E501


        :return: The snapshot_id of this ResponseVolumeSourceDataset.  # noqa: E501
        :rtype: int
        """
        return self._snapshot_id

    @snapshot_id.setter
    def snapshot_id(self, snapshot_id):
        """Sets the snapshot_id of this ResponseVolumeSourceDataset.


        :param snapshot_id: The snapshot_id of this ResponseVolumeSourceDataset.  # noqa: E501
        :type snapshot_id: int
        """

        self._snapshot_id = snapshot_id

    @property
    def sub_path(self):
        """Gets the sub_path of this ResponseVolumeSourceDataset.  # noqa: E501


        :return: The sub_path of this ResponseVolumeSourceDataset.  # noqa: E501
        :rtype: str
        """
        return self._sub_path

    @sub_path.setter
    def sub_path(self, sub_path):
        """Sets the sub_path of this ResponseVolumeSourceDataset.


        :param sub_path: The sub_path of this ResponseVolumeSourceDataset.  # noqa: E501
        :type sub_path: str
        """

        self._sub_path = sub_path

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseVolumeSourceDataset):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseVolumeSourceDataset):
            return True

        return self.to_dict() != other.to_dict()
