# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from savvihub_client.configuration import Configuration


class ModelResourceRequirements(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'guarantee': 'int',
        'types': 'list[str]',
        'version': 'int'
    }

    attribute_map = {
        'guarantee': 'guarantee',
        'types': 'types',
        'version': 'version'
    }

    def __init__(self, guarantee=None, types=None, version=None, local_vars_configuration=None):  # noqa: E501
        """ModelResourceRequirements - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._guarantee = None
        self._types = None
        self._version = None
        self.discriminator = None

        if guarantee is not None:
            self.guarantee = guarantee
        if types is not None:
            self.types = types
        if version is not None:
            self.version = version

    @property
    def guarantee(self):
        """Gets the guarantee of this ModelResourceRequirements.  # noqa: E501


        :return: The guarantee of this ModelResourceRequirements.  # noqa: E501
        :rtype: int
        """
        return self._guarantee

    @guarantee.setter
    def guarantee(self, guarantee):
        """Sets the guarantee of this ModelResourceRequirements.


        :param guarantee: The guarantee of this ModelResourceRequirements.  # noqa: E501
        :type guarantee: int
        """

        self._guarantee = guarantee

    @property
    def types(self):
        """Gets the types of this ModelResourceRequirements.  # noqa: E501


        :return: The types of this ModelResourceRequirements.  # noqa: E501
        :rtype: list[str]
        """
        return self._types

    @types.setter
    def types(self, types):
        """Sets the types of this ModelResourceRequirements.


        :param types: The types of this ModelResourceRequirements.  # noqa: E501
        :type types: list[str]
        """

        self._types = types

    @property
    def version(self):
        """Gets the version of this ModelResourceRequirements.  # noqa: E501


        :return: The version of this ModelResourceRequirements.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ModelResourceRequirements.


        :param version: The version of this ModelResourceRequirements.  # noqa: E501
        :type version: int
        """

        self._version = version

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelResourceRequirements):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ModelResourceRequirements):
            return True

        return self.to_dict() != other.to_dict()
